@@ -24,13 +24,16 @@ try:
     set
 except NameError:
     from sets import Set as set
-# The ``copy`` module was added in Python 2.5 and ``copycompat`` was added in
-# post 1.1.1 Django (r11901)
+# The ``copy`` module became function-friendly in Python 2.5 and
+# ``copycompat`` was added in post 1.1.1 Django (r11901)..
 try:
     from django.utils.copycompat import deepcopy
-    from django.views.decorators.csrf import csrf_exempt
 except ImportError:
     from copy import deepcopy
+# If ``csrf_exempt`` isn't present, stub it.
+try:
+    from django.views.decorators.csrf import csrf_exempt
+except ImportError:
     def csrf_exempt(func):
         return func
 
@@ -157,7 +160,7 @@ class Resource(object):
     def __getattr__(self, name):
         if name in self.fields:
             return self.fields[name]
-        raise AttributeError
+        raise AttributeError(name)
     
     def wrap_view(self, view):
         """
@@ -322,7 +325,49 @@ class Resource(object):
         
         Mostly a hook, this uses the ``Serializer`` from ``Resource._meta``.
         """
-        return self._meta.serializer.deserialize(data, format=request.META.get('CONTENT_TYPE', 'application/json'))
+        deserialized = self._meta.serializer.deserialize(data, format=request.META.get('CONTENT_TYPE', 'application/json'))
+        return deserialized
+    
+    def alter_list_data_to_serialize(self, request, data):
+        """
+        A hook to alter list data just before it gets serialized & sent to the user.
+        
+        Useful for restructuring/renaming aspects of the what's going to be
+        sent.
+        
+        Should accommodate for a list of objects, generally also including
+        meta data.
+        """
+        return data
+    
+    def alter_detail_data_to_serialize(self, request, data):
+        """
+        A hook to alter detail data just before it gets serialized & sent to the user.
+        
+        Useful for restructuring/renaming aspects of the what's going to be
+        sent.
+        
+        Should accommodate for receiving a single bundle of data.
+        """
+        return data
+    
+    def alter_deserialized_list_data(self, request, data):
+        """
+        A hook to alter list data just after it has been received from the user &
+        gets deserialized.
+        
+        Useful for altering the user data before any hydration is applied.
+        """
+        return data
+    
+    def alter_deserialized_detail_data(self, request, data):
+        """
+        A hook to alter detail data just after it has been received from the user &
+        gets deserialized.
+        
+        Useful for altering the user data before any hydration is applied.
+        """
+        return data
     
     def dispatch_list(self, request, **kwargs):
         """
@@ -623,7 +668,8 @@ class Resource(object):
                     if not getattr(field_object, 'is_related', False):
                         setattr(bundle.obj, field_object.attribute, value)
                     elif not getattr(field_object, 'is_m2m', False):
-                        setattr(bundle.obj, field_object.attribute, value.obj)
+                        if value is not None:
+                            setattr(bundle.obj, field_object.attribute, value.obj)
             
             # Check for an optional method to do further hydration.
             method = getattr(self, "hydrate_%s" % field_name, None)
@@ -939,6 +985,7 @@ class Resource(object):
         
         # Dehydrate the bundles in preparation for serialization.
         to_be_serialized['objects'] = [self.full_dehydrate(obj=obj) for obj in to_be_serialized['objects']]
+        to_be_serialized = self.alter_list_data_to_serialize(request, to_be_serialized)
         return self.create_response(request, to_be_serialized)
     
     def get_detail(self, request, **kwargs):
@@ -958,6 +1005,7 @@ class Resource(object):
             return HttpMultipleChoices("More than one resource is found at this URI.")
         
         bundle = self.full_dehydrate(obj)
+        bundle = self.alter_detail_data_to_serialize(request, bundle)
         return self.create_response(request, bundle)
     
     def put_list(self, request, **kwargs):
@@ -970,6 +1018,7 @@ class Resource(object):
         Return ``HttpAccepted`` (204 No Content).
         """
         deserialized = self.deserialize(request, request.raw_post_data, format=request.META.get('CONTENT_TYPE', 'application/json'))
+        deserialized = self.alter_deserialized_list_data(request, deserialized)
         
         if not 'objects' in deserialized:
             raise BadRequest("Invalid data sent.")
@@ -1005,13 +1054,14 @@ class Resource(object):
         If an existing resource is modified, return ``HttpAccepted`` (204 No Content).
         """
         deserialized = self.deserialize(request, request.raw_post_data, format=request.META.get('CONTENT_TYPE', 'application/json'))
+        deserialized = self.alter_deserialized_detail_data(request, deserialized)
         bundle = self.build_bundle(data=dict_strip_unicode_keys(deserialized))
         self.is_valid(bundle, request)
         
         try:
             updated_bundle = self.obj_update(bundle, request=request, pk=kwargs.get('pk'))
             return HttpAccepted()
-        except:
+        except (NotFound, MultipleObjectsReturned):
             updated_bundle = self.obj_create(bundle, request=request, pk=kwargs.get('pk'))
             return HttpCreated(location=self.get_resource_uri(updated_bundle))
     
@@ -1025,6 +1075,7 @@ class Resource(object):
         If a new resource is created, return ``HttpCreated`` (201 Created).
         """
         deserialized = self.deserialize(request, request.raw_post_data, format=request.META.get('CONTENT_TYPE', 'application/json'))
+        deserialized = self.alter_deserialized_list_data(request, deserialized)
         bundle = self.build_bundle(data=dict_strip_unicode_keys(deserialized))
         self.is_valid(bundle, request)
         updated_bundle = self.obj_create(bundle, request=request)
@@ -1261,6 +1312,48 @@ class ModelResource(Resource):
         
         return final_fields
     
+    def check_filtering(self, field_name, filter_type='exact', filter_bits=None):
+        """
+        Given a field name, a optional filter type and an optional list of
+        additional relations, determine if a field can be filtered on.
+        
+        If a filter does not meet the needed conditions, it should raise an
+        ``InvalidFilterError``.
+        
+        If the filter meets the conditions, a list of attribute names (not
+        field names) will be returned.
+        """
+        if filter_bits is None:
+            filter_bits = []
+        
+        if not field_name in self._meta.filtering:
+            raise InvalidFilterError("The '%s' field does not allow filtering." % field_name)
+        
+        # Check to see if it's an allowed lookup type.
+        if not self._meta.filtering[field_name] in (ALL, ALL_WITH_RELATIONS):
+            # Must be an explicit whitelist.
+            if not filter_type in self._meta.filtering[field_name]:
+                raise InvalidFilterError("'%s' is not an allowed filter on the '%s' field." % (filter_type, field_name))
+        
+        if self.fields[field_name].attribute is None:
+            raise InvalidFilterError("The '%s' field has no 'attribute' for searching with." % field_name)
+        
+        # Check to see if it's a relational lookup and if that's allowed.
+        if len(filter_bits):
+            if not getattr(self.fields[field_name], 'is_related', False):
+                raise InvalidFilterError("The '%s' field does not support relations." % field_name)
+            
+            if not self._meta.filtering[field_name] == ALL_WITH_RELATIONS:
+                raise InvalidFilterError("Lookups are not allowed more than one level deep on the '%s' field." % field_name)
+            
+            # Recursively descend through the remaining lookups in the filter,
+            # if any. We should ensure that all along the way, we're allowed
+            # to filter on that field by the related resource.
+            related_resource = self.fields[field_name].get_related_resource(None)
+            return [self.fields[field_name].attribute] + related_resource.check_filtering(filter_bits[0], filter_type, filter_bits[1:])
+        
+        return [self.fields[field_name].attribute]
+    
     def build_filters(self, filters=None):
         """
         Given a dictionary of filters, create the necessary ORM-level filters.
@@ -1287,32 +1380,17 @@ class ModelResource(Resource):
         
         for filter_expr, value in filters.items():
             filter_bits = filter_expr.split(LOOKUP_SEP)
+            field_name = filter_bits.pop(0)
+            filter_type = 'exact'
             
-            if not filter_bits[0] in self.fields:
+            if not field_name in self.fields:
                 # It's not a field we know about. Move along citizen.
                 continue
             
-            if not filter_bits[0] in self._meta.filtering:
-                raise InvalidFilterError("The '%s' field does not allow filtering." % filter_bits[0])
-            
-            if filter_bits[-1] in QUERY_TERMS.keys():
+            if len(filter_bits) and filter_bits[-1] in QUERY_TERMS.keys():
                 filter_type = filter_bits.pop()
-            else:
-                filter_type = 'exact'
             
-            # Check to see if it's allowed lookup type.
-            if not self._meta.filtering[filter_bits[0]] in (ALL, ALL_WITH_RELATIONS):
-                # Must be an explicit whitelist.
-                if not filter_type in self._meta.filtering[filter_bits[0]]:
-                    raise InvalidFilterError("'%s' is not an allowed filter on the '%s' field." % (filter_expr, filter_bits[0]))
-            
-            # Check to see if it's a relational lookup and if that's allowed.
-            if len(filter_bits) > 1:
-                if not self._meta.filtering[filter_bits[0]] == ALL_WITH_RELATIONS:
-                    raise InvalidFilterError("Lookups are not allowed more than one level deep on the '%s' field." % filter_bits[0])
-            
-            if self.fields[filter_bits[0]].attribute is None:
-                raise InvalidFilterError("The '%s' field has no 'attribute' for searching with." % filter_bits[0])
+            lookup_bits = self.check_filtering(field_name, filter_type, filter_bits)
             
             if value in ['true', 'True', True]:
                 value = True
@@ -1321,7 +1399,11 @@ class ModelResource(Resource):
             elif value in ('nil', 'none', 'None', None):
                 value = None
             
-            db_field_name = LOOKUP_SEP.join([self.fields[filter_bits[0]].attribute] + filter_bits[1:])
+            # Split on ',' if not empty string and either an in or range filter.
+            if filter_type in ('in', 'range') and len(value):
+                value = value.split(',')
+            
+            db_field_name = LOOKUP_SEP.join(lookup_bits)
             qs_filter = "%s%s%s" % (db_field_name, LOOKUP_SEP, filter_type)
             qs_filters[qs_filter] = value
         
@@ -1388,15 +1470,9 @@ class ModelResource(Resource):
         """
         An ORM-specific implementation of ``get_object_list``.
         
-        Returns a queryset that may have been limited by authorization or other
-        overrides.
+        Returns a queryset that may have been limited by other overrides.
         """
-        base_object_list = self._meta.queryset
-        
-        # Limit it as needed.
-        authed_object_list = self.apply_authorization_limits(request, base_object_list)
-        
-        return authed_object_list
+        return self._meta.queryset._clone()
     
     def obj_get_list(self, request=None, **kwargs):
         """
@@ -1416,9 +1492,10 @@ class ModelResource(Resource):
         applicable_filters = self.build_filters(filters=filters)
         
         try:
-            return self.get_object_list(request).filter(**applicable_filters)
+            base_object_list = self.get_object_list(request).filter(**applicable_filters)
+            return self.apply_authorization_limits(request, base_object_list)
         except ValueError, e:
-            raise NotFound("Invalid resource lookup data provided (mismatched type).")
+            raise BadRequest("Invalid resource lookup data provided (mismatched type).")
     
     def obj_get(self, request=None, **kwargs):
         """
@@ -1428,7 +1505,16 @@ class ModelResource(Resource):
         the instance.
         """
         try:
-            return self.get_object_list(request).get(**kwargs)
+            base_object_list = self.get_object_list(request).filter(**kwargs)
+            object_list = self.apply_authorization_limits(request, base_object_list)
+            stringified_kwargs = ', '.join(["%s=%s" % (k, v) for k, v in kwargs.items()])
+            
+            if len(object_list) <= 0:
+                raise self._meta.object_class.DoesNotExist("Couldn't find an instance of '%s' which matched '%s'." % (self._meta.object_class.__name__, stringified_kwargs))
+            elif len(object_list) > 1:
+                raise MultipleObjectsReturned("More than '%s' matched '%s'." % (self._meta.object_class.__name__, stringified_kwargs))
+            
+            return object_list[0]
         except ValueError, e:
             raise NotFound("Invalid resource lookup data provided (mismatched type).")
     
@@ -1471,7 +1557,7 @@ class ModelResource(Resource):
                 # and this will work fine.
                 lookup_kwargs = kwargs
             try:
-                bundle.obj = self.get_object_list(request).get(**lookup_kwargs)
+                bundle.obj = self.obj_get(request, **lookup_kwargs)
             except ObjectDoesNotExist:
                 raise NotFound("A model instance matching the provided arguments could not be found.")
         
@@ -1489,7 +1575,15 @@ class ModelResource(Resource):
         
         Takes optional ``kwargs``, which can be used to narrow the query.
         """
-        self.get_object_list(request).filter(**kwargs).delete()
+        base_object_list = self.get_object_list(request).filter(**kwargs)
+        authed_object_list = self.apply_authorization_limits(request, base_object_list)
+        
+        if hasattr(authed_object_list, 'delete'):
+            # It's likely a ``QuerySet``. Call ``.delete()`` for efficiency.
+            authed_object_list.delete()
+        else:
+            for authed_obj in authed_object_list:
+                authed_object_list.delete()
     
     def obj_delete(self, request=None, **kwargs):
         """
@@ -1499,7 +1593,7 @@ class ModelResource(Resource):
         the instance.
         """
         try:
-            obj = self.get_object_list(request).get(**kwargs)
+            obj = self.obj_get(request, **kwargs)
         except ObjectDoesNotExist:
             raise NotFound("A model instance matching the provided arguments could not be found.")
         
